# $Id: MoviePosterDB.pm 3636 2010-01-10 19:10:47Z chris $

=head1 NAME

WebService::MoviePosterDB - OO Perl interface to the movie poster database MoviePosterDB.

=head1 SYNOPSIS

    use WebService::MoviePosterDB;

    my $ws = WebService::MoviePosterDB->new(cache => 1, cache_exp => "12h");

    my $movie = $ws->search(imdb_code => "0114814", width => 300);

    print $movie->title(), ": \n\n";
    print $movie->page(), "\n\n";

    foreach ( @{$movie->posters()} ) {
        print $_->image_location(), "\n";
    }

=head1 DESCRIPTION

WebService::MusicBrainz is an object-oriented interface to MoviePosterDB.  It can 
be used to retrieve artwork for IMDB titles.

=cut

package WebService::MoviePosterDB;

use strict;
use warnings;

our $VERSION = '0.05';

use Cache::FileCache;

use File::Spec::Functions qw(tmpdir);

use JSON;
use LWP::UserAgent;
use URI;

use WebService::MoviePosterDB::Movie;

=head1 METHODS

=head2 new(%opts)

Constructor.

%opts can contain:

=over 4

=item cache - Whether to cache responses.  Defaults to true

=item cache_root - The root dir for the cache.  Defaults to tmpdir();

=item cache_exp - How long to cache responses for.  Defaults to "1h"

=back

=cut

sub new {
    my $class = shift;
    my %args = @_;
    my $self = {};
    
    bless $self, $class;
    
    $self->{'api_key'} = $args{'api_key'} || "demo";
    $self->{'api_secret'} = $args{'api_secret'} || "demo";

    $self->{'_cache_root'} = $args{'cache_root'} || tmpdir();
    $self->{'_cache_exp'} = $args{'cache_exp'} || "1h";
    $self->{'_cache'} = defined $args{'cache'} ? $args{'cache'} : 1;

    if ($self->{'_cache'}) {
	$self->{'_cacheObj'} = Cache::FileCache->new( {'cache_root' => $self->{'_cache_root'}, 'namespace' => "WebService-MoviePosterDB", 'default_expires_in' => $self->{'_cache_exp'}} );
    }

    $self->{'_useragent'} = LWP::UserAgent->new();
    $self->{'_useragent'}->env_proxy();
    $self->{'_useragent'}->agent("WebService::MoviePosterDB/$VERSION");
    
    return $self;
}


=head2 search(%args)

Accesses MoviePosterDB and returns a WebService::MoviePosterDB::Movie object.

%args can contain:

=over 4

=item imdb_code - IMDB code for the title

=item title - Name of the title

=item width - Image width for returned artwork

=back

=cut

# In the fullness of time, the intention is to use the proper MoviePosterDB api, 
# accessed via http://api.movieposterdb.com/json, but until the mechanism for
# generating the secret hash is known, we use the old api, 
# http://api.movieposterdb.com/json.inc.php, and translate the parameters.

sub search {
    my $self = shift;
    my %args = @_;
    
    my $web_service_uri = URI->new();

    $web_service_uri->scheme("http");
    $web_service_uri->host("api.movieposterdb.com");
    $web_service_uri->path("json.inc.php");
    
    my $url = $web_service_uri->as_string();

    {
	# TODO: Validate these against a permitted list + perform url encoding as appropriate
	my $f = 1;
	foreach (keys %args) {
	    my ($k, $v) = ($_, $args{$_});
	    
	    # Fixup to make json.inc.php look like json
	    if ($k eq "imdb_code") {
		$k = "imdb";
	    } elsif ($k eq "title" || $k eq "width") {
		# Nothing
	    } else {
		next;
	    }

	    $url .= ($f ? "?" : "&") . $k . "=" . $v;
	    undef $f;
	}
    }

    my $content;

    if ($self->{'_cache'}) {
	$content = $self->{'_cacheObj'}->get($url);
    }

    if (! defined $content) {
	my $response = $self->{'_useragent'}->get($url);
    
	if($response->code() ne "200") {
	    die "URL (", $url, ") Request Failed - Code: ", $response->code(), " Error: ", $response->message(), "\n";
	}

	$content = $response->decoded_content();

	if ($self->{'_cache'}) {
	    $self->{'_cacheObj'}->set($url, $content);
	}
    }
    
    my $json = new JSON;

    return WebService::MoviePosterDB::Movie->_new($json->decode($content));


}

1;
